/* bin/cli.js
 * Originally created 11/13/2018 by sapeck
 *
 * Copyright 2018-present sapeck
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
'use strict'

import isRoot from 'is-root'
import sourceMapSupport from 'source-map-support'
import yargs from 'yargs'

import * as filterRules from '../lib/filterRules.json'
import replaceList from '../lib/replaceList.json'
import File from '../lib/file'
import Filter from '../lib/filter'
import Random from '../lib/random'
import Replace from '../lib/replace'
import Order from '../lib/order'

if (process.env.NODE_ENV === 'development') sourceMapSupport.install()

// No root!
if (isRoot()) {
  console.error(new Error('Never run this script as root!'))
  process.exit(1)
}

// eslint-disable-next-line no-unused-expressions
yargs
  .command('filter <path> <out> [maxLength]', 'apply filter to file', (yargs) => {
    yargs
      .positional('path', {
        type: 'string',
        describe: 'path to file to filter'
      })
      .positional('out', {
        type: 'string',
        describe: 'path to the output file'
      })
      .positional('maxLength', {
        type: 'number',
        default: 0,
        describe: 'maximum length of a line'
      })
  }, async function (argv) {
    let inFile = new File(argv.path)
    let filter = new Filter(filterRules)
    let filtered = await filter.filter(inFile, argv.maxLength)
    let outFile = new File(argv.out, true)
    outFile.setContents(filtered)
  })
  .command('filterstart <path> <out>', 'apply start filter to file', (yargs) => {
    yargs
      .positional('path', {
        type: 'string',
        describe: 'path to file to filter'
      })
      .positional('out', {
        type: 'string',
        describe: 'path to the output file'
      })
  }, async function (argv) {
    let inFile = new File(argv.path)
    let filter = new Filter(filterRules)
    let filteredStart = await filter.filterStart(inFile)
    let outFile = new File(argv.out, true)
    outFile.setContents(filteredStart)
  })
  .command('random <path> <out> <count>', 'randomly select from a file', (yargs) => {
    yargs
      .positional('path', {
        type: 'string',
        describe: 'path to file to select from'
      })
      .positional('out', {
        type: 'string',
        describe: 'path to the output file'
      })
      .positional('count', {
        type: 'number',
        describe: 'number of lines to return'
      })
  }, async function (argv) {
    let inFile = new File(argv.path)
    let randomed = Random.random(inFile, argv.count)
    let outFile = new File(argv.out, true)
    outFile.setContents(randomed)
  })
  .command('replace <path> <out>', 'apply replacements to file', (yargs) => {
    yargs
      .positional('path', {
        type: 'string',
        describe: 'path to file to replace in'
      })
      .positional('out', {
        type: 'string',
        describe: 'path to the output file'
      })
  }, async function (argv) {
    let inFile = new File(argv.path)
    let replace = new Replace(replaceList)
    let replaced = await replace.replace(inFile)
    let outFile = new File(argv.out, true)
    outFile.setContents(replaced)
  })
  .command('replacepeople <path> <out>', 'apply people replacements to file', (yargs) => {
    yargs
      .positional('path', {
        type: 'string',
        describe: 'path to file to replace in'
      })
      .positional('out', {
        type: 'string',
        describe: 'path to the output file'
      })
  }, async function (argv) {
    let inFile = new File(argv.path)
    let replacedPeople = await Replace.replacePeople(inFile)
    let outFile = new File(argv.out, true)
    outFile.setContents(replacedPeople)
  })
  .command('order <path> <out>', 'reorder a file', (yargs) => {
    yargs
      .positional('path', {
        type: 'string',
        describe: 'path to file to order in'
      })
      .positional('out', {
        type: 'string',
        describe: 'path to the output file'
      })
  }, async function (argv) {
    let inFile = new File(argv.path)
    let ordered = await Order.order(inFile)
    let outFile = new File(argv.out, true)
    outFile.setContents(ordered)
  })
  .demandCommand(1, 1)
  .help()
  .argv
